/**********************************************************************
 *
 * Copyright (c) 2004 Olaf Willuhn
 * All rights reserved.
 * 
 * This software is copyrighted work licensed under the terms of the
 * Jameica License.  Please consult the file "LICENSE" for details. 
 *
 **********************************************************************/

package de.willuhn.jameica.fibu.search;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.fibu.Fibu;
import de.willuhn.jameica.fibu.Settings;
import de.willuhn.jameica.fibu.gui.action.BuchungNeu;
import de.willuhn.jameica.fibu.rmi.Buchung;
import de.willuhn.jameica.fibu.rmi.Geschaeftsjahr;
import de.willuhn.jameica.fibu.rmi.Konto;
import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
import de.willuhn.jameica.search.Result;
import de.willuhn.jameica.search.SearchProvider;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Suchprovider fuer Buchungen.
 */
public class BuchungSearchProvider implements SearchProvider
{
  private Geschaeftsjahr jahr = null;

  /**
   * @see de.willuhn.jameica.search.SearchProvider#getName()
   */
  public String getName()
  {
    return Application.getPluginLoader().getPlugin(Fibu.class).getResources().getI18N().tr("Buchungen");
  }

  /**
   * @see de.willuhn.jameica.search.SearchProvider#search(java.lang.String)
   */
  public List search(String search) throws RemoteException,
      ApplicationException
  {
    if (search == null || search.length() == 0)
      return null;

    this.jahr = Settings.getActiveGeschaeftsjahr();
    if (this.jahr == null)
      return null; // Kein Geschaeftsjahr aktiv
    
    DBIterator list = this.jahr.getHauptBuchungen(true);
    list.setOrder("order by belegnummer desc");
    list.addFilter("LOWER(buchungstext) LIKE ?","%" + search.toLowerCase() + "%");

    ArrayList results = new ArrayList();
    while (list.hasNext())
    {
      results.add(new MyResult((Buchung)list.next()));
    }
    return results;
  }
  
  /**
   * Hilfsklasse fuer die formatierte Anzeige der Ergebnisse.
   */
  private class MyResult implements Result
  {
    private Buchung buchung = null;
    
    /**
     * ct.
     * @param buchung
     */
    private MyResult(Buchung buchung)
    {
      this.buchung = buchung;
    }

    /**
     * @see de.willuhn.jameica.search.Result#execute()
     */
    public void execute() throws RemoteException, ApplicationException
    {
      new BuchungNeu().handleAction(this.buchung);
    }

    /**
     * @see de.willuhn.jameica.search.Result#getName()
     */
    public String getName()
    {
      try
      {
        I18N i18n = Application.getPluginLoader().getPlugin(Fibu.class).getResources().getI18N();

        Date datum    = buchung.getDatum();
        Konto soll    = buchung.getSollKonto();
        Konto haben   = buchung.getHabenKonto();
        String text   = buchung.getText();
        int nummer    = buchung.getBelegnummer();
        double betrag = buchung.getBetrag();
        
        return i18n.tr("#{0} vom {1}: {2} an {3}  {4} ({5})", new String[]{Integer.toString(nummer),
                                                                           Settings.DATEFORMAT.format(datum),
                                                                           soll.getKontonummer(),
                                                                           haben.getKontonummer(),
                                                                           new CurrencyFormatter(jahr.getMandant().getWaehrung(),Settings.DECIMALFORMAT).format(Double.valueOf(betrag)),
                                                                           text});
      }
      catch (RemoteException re)
      {
        Logger.error("unable to determin result name",re);
        return null;
      }
    }
    
  }
}
